/*
 *    NzdlService.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

import java.util.Set;
import java.util.Map;
import java.util.List;

/**
 * An object that services CORBA client requests to a Greenstone Digital
 * Library.
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @author Aziz Mahoui    (amahoui@yahoo.com)
 * @author Gordon Paynter (paynter@cs.waikato.ac.nz)
 * @version $Revision: 2159 $
 */
public interface NzdlService {

  /**
   * Allow the client to configure a CORBA server.
   * @param key the parameter to be set.
   * @param values what that parameter will be set to.
   */
  public void configure( String key, Set values );

  /**
   * Obtain the set of collection names for a Greenstone library.
   * @return A set of collection names in string format.
   */
  public Set getCollectionSet( );

  /**
   * Check if ORB knows of this collection.
   * @param collectionName the name of a Greenstone collection.
   * @return boolean <TT>true</TT> if collection is found, else <TT>false</TT>.
   */
  public boolean hasCollection( String collectionName );

  /**
   * Check if ORB can communicate with of this collection.
   * @param collectionName the name of a collection to be pinged.
   * @return boolean <TT>true</TT> if collection returned ping, else
   * <TT>false</TT>.
   */
  public boolean pingCollection( String collectionName );

 /**
   * Extract information on this collection into a
   * {@link NzdlCollectionInfo NzdlCollectionInfo} object
   * @param collectionName the Greenstone collection to be retrieved.
   * @return an instance of
   * {@link NzdlCollectionInfo NzdlCollectionInfo} containing data on a
   * collection
   */
  public NzdlCollectionInfo getCollectionInfo( String collectionName );

  /**
   * Returns the set of filters for the service object.
   * @param collectionName the name of the Greenstone collection.
   * @return a set of strings that name the filters, for example
   * BrowseFilter, QueryFilter and NullFilter.
   */
  public Set getFilterSet( String collectionName );

  /**
   * Not Currently implemented.
   */
  public Set getMetaTagSet( String collectionName );

  /**
   * Returns the Greenstone Markup Language (GML) text for a particular
   * document from a collection.
   * @param collectionName the name of the Greenstone collection.
   * @param documentID the document identity string
   * @return the GML document in the form of a string.
   */
  public String getDocument( String collectionName, String documentID );

  /**
   * Returns the set of value(s) for a metatag from a particular document.
   * @param collectionName the name of the Greenstone collection.
   * @param documentID the document identifier.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The set of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Set getMetaData( String collectionName, String documentID, String metaTag );

  /**
   * Returns a map of value(s) for a metatag from a list of documents.
   * @param collectionName name of the Greenstone collection
   * @param docIDs a list of document identifier strings.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The map of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Map getMetaData( String collectionName, List documentIDs, String metaTag );

  /**
   * Services a {@link NzdlRequest NzdlRequest} to a collection and places
   * the results into a {@link NzdlResponse NzdlResponse} object. <br>
   * Pre:   Created instances of NzdlRequest and NzdlResponse. <br>
   * Post: Placed the result data into NzdlResponse object.
   * @param collectionName name of the Greenstone collection.
   * @param request an instance of NzdlRequest with query data to be serviced
   * @param response an instance of NzdlResponse. where the results of
   * sevciving the query will be placed
   */
  public void service( String collectionName, NzdlRequest request, NzdlResponse response );

}
